Package: ggfittext
Title: Fit Text Inside a Box in 'ggplot2'
Version: 0.10.4
Authors@R: c(person("David", "Wilkins", email = "david@wilkox.org",
                  role = c("aut", "cre")), 
             person("Brady", "Johnston", email = "brady.johnston@me.com", 
                  role = c("ctb"))
             )
Description: A 'ggplot2' extension to fit text into a box by growing, shrinking or wrapping the text.
Depends: R (>= 3.6)
License: GPL-2
LazyData: true
Imports: grid (>= 3.1), stringi (>= 1.1.2), shades (>= 1.3.1), gridtext
        (>= 0.1.4), ggplot2 (>= 2.2.1), cli (>= 3.6.1)
URL: https://wilkox.org/ggfittext/
BugReports: https://github.com/wilkox/ggfittext/issues/
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), vdiffr, spelling
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-GB
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-05 01:45:37 UTC; wilkox
Author: David Wilkins [aut, cre],
  Brady Johnston [ctb]
Maintainer: David Wilkins <david@wilkox.org>
Repository: CRAN
Date/Publication: 2026-07-05 06:40:02 UTC
