# Generated by roxygen2: do not edit by hand

S3method(getfontsize,richtext_grob)
S3method(getfontsize,text)
S3method(getlabel,richtext_grob)
S3method(getlabel,text)
S3method(getrot,richtext_grob)
S3method(getrot,text)
S3method(makeContent,fittexttree)
S3method(makeContent,fittexttreepolar)
S3method(setfontsize,richtext_grob)
S3method(setfontsize,text)
S3method(setlabel,richtext_grob)
S3method(setlabel,text)
S3method(setx,richtext_grob)
S3method(setx,text)
S3method(sety,richtext_grob)
S3method(sety,text)
S3method(wraplabel,richtext_grob)
S3method(wraplabel,text)
export(geom_bar_text)
export(geom_fit_text)
importFrom(grid,makeContent)
