% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{gold}
\alias{gold}
\title{Robert Frost's poem \emph{Nothing Gold Can Stay} (1923)}
\format{
A data frame with 8 rows and 6 variables:
\describe{
\item{line}{a line from the poem}
\item{xmin}{the xmin coordinate with which to draw the line}
\item{xmax}{the xmax coordinate with which to draw the line}
\item{ymin}{the ymin coordinate with which to draw the line}
\item{ymax}{the ymax coordinate with which to draw the line}
\item{linenumber}{the line number}
}
}
\usage{
gold
}
\description{
Robert Frost's poem \emph{Nothing Gold Can Stay} (1923)
}
\keyword{datasets}
